import { useEffect, useState } from 'react';
import { KeyParse, getTips } from './key-parse';
import { FormControlLabel, TextField, Select, MenuItem, FormGroup, Tooltip } from '@mui/material';
import { DatePicker } from '../draw/modules/DatePicker';
import { SelectPicker } from '../draw/modules/SelectPicker';
import { HelpCircle } from 'lucide-react';
import clsx from 'clsx';
import { useTranslation } from 'react-i18next';
export const KeyShareSelect = ({ name, value, onChange }: { name: string; value: string; onChange?: (value: string) => void }) => {
  return (
    <Select
      variant='outlined'
      size='small'
      name={name}
      value={value || ''}
      onChange={(e) => onChange?.(e.target.value)}
      sx={{
        width: '100%',
        marginBottom: '16px',
      }}>
      <MenuItem value='public' title='公开'>
        公开
      </MenuItem>
      <MenuItem value='protected' title='受保护'>
        受保护
      </MenuItem>
      <MenuItem value='private' title='私有'>
        私有
      </MenuItem>
    </Select>
  );
};
export const KeyTextField = ({ name, value, onChange }: { name: string; value: string; onChange?: (value: string) => void }) => {
  return (
    <TextField
      variant='outlined'
      size='small'
      name={name}
      value={value}
      onChange={(e) => onChange?.(e.target.value)}
      sx={{
        width: '100%',
        marginBottom: '16px',
      }}
    />
  );
};

type PermissionManagerProps = {
  value: Record<string, any>;
  onChange: (value: Record<string, any>) => void;
  className?: string;
};
export const PermissionManager = ({ value, onChange, className }: PermissionManagerProps) => {
  const { t, i18n } = useTranslation();
  const [formData, setFormData] = useState<any>(value);
  const [keys, setKeys] = useState<any>([]);
  useEffect(() => {
    const hasShare = value?.share && value?.share === 'protected';
    setFormData(KeyParse.parse(value || {}));
    if (hasShare) {
      setKeys(['password', 'usernames', 'expiration-time']);
    } else {
      setKeys([]);
    }
  }, [value]);
  const onChangeValue = (key: string, newValue: any) => {
    // setFormData({ ...formData, [key]: newValue });
    let newFormData = { ...formData, [key]: newValue };
    if (key === 'share') {
      if (newValue === 'protected') {
        newFormData = { ...newFormData, password: '', usernames: [], 'expiration-time': null };
        onChange(KeyParse.stringify(newFormData));
        setKeys(['password', 'usernames', 'expiration-time']);
      } else {
        delete newFormData.password;
        delete newFormData.usernames;
        delete newFormData['expiration-time'];
        onChange(KeyParse.stringify(newFormData));
        setKeys([]);
      }
    } else {
      onChange(KeyParse.stringify(newFormData));
    }
  };
  const tips = getTips('share', i18n.language);
  return (
    <form className={clsx('flex flex-col gap-2', className)}>
      <FormControlLabel
        control={<KeyShareSelect name='share' value={formData?.share} onChange={(value) => onChangeValue('share', value)} />}
        label={
          <div className='flex items-center gap-1'>
            {t('Share')}
            <Tooltip title={tips}>
              <HelpCircle size={16} />
            </Tooltip>
          </div>
        }
      />
      {keys.map((item: any) => {
        let control: React.ReactNode | null = null;
        if (item === 'expiration-time') {
          control = <DatePicker value={formData[item] || ''} onChange={(date) => onChangeValue(item, date)} />;
        } else if (item === 'usernames') {
          control = <SelectPicker value={formData[item] || []} onChange={(value) => onChangeValue(item, value)} />;
        } else {
          control = <KeyTextField name={item} value={formData[item] || ''} onChange={(value) => onChangeValue(item, value)} />;
        }
        const tips = getTips(item);
        return (
          <FormControlLabel
            labelPlacement='top'
            key={item}
            control={control}
            label={
              <div className='flex items-center gap-1'>
                {item}
                {tips && (
                  <Tooltip title={tips}>
                    <HelpCircle size={16} />
                  </Tooltip>
                )}
              </div>
            }
            sx={{
              alignItems: 'flex-start',
              '& .MuiFormControlLabel-label': {
                textAlign: 'left',
                width: '100%',
              },
            }}
          />
        );
      })}
    </form>
  );
};
